import graphene
from graphene_django import DjangoObjectType
from graphene_django_cud.mutations import DjangoCreateMutation, DjangoPatchMutation

from aleksis.core.models import Group, Person, Room, CalendarEvent
from ..models import LessonEvent

from ..util.chronos_helpers import get_classes, get_rooms, get_teachers


class TimetablePersonType(DjangoObjectType):
    class Meta:
        model = Person
        fields = ("id", "first_name", "last_name", "short_name")
        skip_registry = True


class TimetableGroupType(DjangoObjectType):
    class Meta:
        model = Group
        fields = ("id", "name", "short_name")
        skip_registry = True


class TimetableRoomType(DjangoObjectType):
    class Meta:
        model = Room
        fields = ("id", "name", "short_name")
        skip_registry = True


# There is another unrelated CalendarEventType in aleksis/core/schema/calendar
# This CalendarEventType is needed for the inherited amends field of LessonEvent
# to work in the graphql query.
class CalendarEventForLessonEventType(DjangoObjectType):
    class Meta:
        model = CalendarEvent
        fields = ("id",
                  "amends",
                  "datetime_start",
                  "datetime_end")


class LessonEventType(DjangoObjectType):
    class Meta:
        model = LessonEvent
        fields = ("id",
                  "amends",
                  "datetime_start",
                  "datetime_end",
                  "subject",
                  "teachers",
                  "groups",
                  "rooms",
                  "cancelled",
                  "comment")


class AmendLessonCreateMutation(DjangoCreateMutation):
    class Meta:
        model = LessonEvent
        permissions = ("chronos.edit_substitution_rule",)
        only_fields = ("amends",
                       "datetime_start",
                       "datetime_end",
                       "subject",
                       "teachers",
                       "groups",
                       "rooms",
                       "cancelled",
                       "comment")


class AmendLessonPatchMutation(DjangoPatchMutation):
    class Meta:
        model = LessonEvent
        permissions = ("chronos.edit_substitution_rule",)
        only_fields = (
                       "subject",
                       "teachers",
                       "groups",
                       "rooms",
                       "cancelled",
                       "comment")


class TimetableType(graphene.Enum):
    TEACHER = "teacher"
    GROUP = "group"
    ROOM = "room"


class TimetableObjectType(graphene.ObjectType):
    id = graphene.String()  # noqa
    obj_id = graphene.String()
    name = graphene.String()
    short_name = graphene.String()
    type = graphene.Field(TimetableType)  # noqa

    def resolve_obj_id(root, info, **kwargs):
        return root.id

    def resolve_id(root, info, **kwargs):
        return f"{root.type.value}-{root.id}"


class Query(graphene.ObjectType):
    timetable_teachers = graphene.List(TimetablePersonType)
    timetable_groups = graphene.List(TimetableGroupType)
    timetable_rooms = graphene.List(TimetableRoomType)
    available_timetables = graphene.List(TimetableObjectType)

    def resolve_timetable_teachers(self, info, **kwargs):
        return get_teachers(info.context.user)

    def resolve_timetable_groups(self, info, **kwargs):
        return get_classes(info.context.user)

    def resolve_timetable_rooms(self, info, **kwargs):
        return get_rooms(info.context.user)

    def resolve_available_timetables(self, info, **kwargs):
        all_timetables = []
        for group in get_classes(info.context.user):
            all_timetables.append(
                TimetableObjectType(
                    id=group.id,
                    name=group.name,
                    short_name=group.short_name,
                    type=TimetableType.GROUP,
                )
            )

        for teacher in get_teachers(info.context.user):
            print(teacher.full_name)
            all_timetables.append(
                TimetableObjectType(
                    id=teacher.id,
                    name=teacher.full_name,
                    short_name=teacher.short_name,
                    type=TimetableType.TEACHER,
                )
            )

        for room in get_rooms(info.context.user):
            all_timetables.append(
                TimetableObjectType(
                    id=room.id, name=room.name, short_name=room.short_name, type=TimetableType.ROOM
                )
            )

        return all_timetables


class Mutation(graphene.ObjectType):
    create_amend_lesson = AmendLessonCreateMutation.Field()
    patch_amend_lesson = AmendLessonPatchMutation.Field()
